<?php

/**
 * Clientix Soap Client
 * Client Optimized to handle sessions and calls for the clientix methods
 * For more information contact  support@alenet.com
 *
 * @author       $Author: imrsnik $
 * @copyright    Alenet, Inc.
 * @since        $Date:  16:30:00$
 * @version      $Rev: 1$
 * @package		Clientix.Classes.Webservice
 * @abstract		Abstract Class
 */
class clx_soap_client {

    protected $url = false; //Servers URL
    protected $wsdl = false;       //Server URL to WSDL
    protected $dbn = CLX_INIT_ZERO;     //System to connect to
    protected $language = 'en';    //Language
    protected $cnx;  //Soap Object
    protected $sessionid;   //Client Session ID
    protected $errors = array('Error, try connect() before calling any method');
    protected $loggedin = false;

    /**
     * Constructor
     * @param $server_url Server url
     * @param $lang		  Language to use on session en (default),es,es_PA,etc
     * @param ClxCommon::$dbn 		  Server system to connecto to default 0
     * @param $options	  Soap client options as defines on php SoapClient
     */
    public function __construct($server_url, $lang = 'en', $dbn = CLX_NUM_ZERO, $options = array()) {
        $this->lang = clx_language_lower_upper($lang);
        $this->dbn = ClxCommon::$dbn;
        $this->url = $server_url;
        $this->wsdl = $server_url . '?wsdl=1&dbn=' . $this->dbn;
        $this->options = $options;
        $this->options['encoding'] = 'UTF-8'; //setting client encoding
        $this->options['trace'] = CLX_INIT_ZERO;
        $this->sessionid = '0';
        $this->cnx = false;
    }

    /**
     * Distruct Method
     */
    public function __destruct() {
        $this->logout();
    }

    /**
     * Adds error messages
     * @param string $error_key
     * @param string $error_msg
     */
    public function add_error_msg($error_key, $error_msg) {
        $this->errors[$error_key] = $error_msg;
    }

    /**
     * Format Error messages for display
     * @param string $key
     * @param string $value
     * @return multitype:string |string
     */
    private function format_error_msg($key, $value) {
        if (array_key_exists($key, $this->errors)) {
            return $this->errors[$key];
        } else {
            return '<li>' . utf8_encode($key) . ': ' . utf8_encode($value) . '</li>';
        }
    }

    /**
     * List all methods defined by the wsdl
     * @param bool $dump	If set to true, methods will be dump into screen
     */
    public function listMethods($dump = false) {
        $contenidoHTML = '';
        if (empty($this->cnx))
            salir($this->errors[CLX_NUM_ZERO]);
        try {
            $methods = $this->cnx->__getFunctions();
        } catch (Exception $e) {
            salir($e->getMessage());
        }
        if (!empty($dump)) {
            $contenidoHTML.= '<ol><li>' . implode($methods, '</li><li>') . '</li><ol>';
        } else {
            echo $contenidoHTML;
            return $methods;
        }
        echo $contenidoHTML;
    }

    /**
     * Creates Soap Object and
     * @param bool $wsdl_cache	Sets to true to cache wsdl
     */
    public function connect($wsdl_cache = CLX_NUM_ONE) {
        if ($wsdl_cache === true) {
            $this->options['cache_wsdl'] = CLX_NUM_ONE;
            ini_set('soap.wsdl_cache_enabled', CLX_NUM_ONE);
        } else {
            $this->options['cache_wsdl'] = WSDL_CACHE_NONE;
            ini_set('soap.wsdl_cache_enabled', CLX_NUM_ZERO);
        }
        //build client
        try {
            $this->cnx = new SoapClient($this->wsdl, $this->options);
        } catch (Exception $e) {
            salir($e->getMessage());
        }
        //starting cookie
        $this->startSession();
    }

    /**
     * Logs out, terminates session and disconnect soap client
     */
    public function disconnect() {
        $this->endSession();
        $this->cnx = false;
    }

    /**
     * Performs the webservice method call
     * @param string $methodname	Method name
     * @param array $parameters 	Optional method parameters
     * @return mixed $response
     */
    public function call($methodname, $parameters = array()) {
        $contenidoHTML = '';
        if (empty($this->cnx))
            echo $contenidoHTML;
        salir($this->errors[CLX_NUM_ZERO]);
        try {
            echo $contenidoHTML;
            return $this->cnx->__call($methodname, $parameters);
        } catch (Exception $e) {
            $contenidoHTML.= '<b>' . $e->getMessage() . '</b>';
            if (!empty($parameters)) {
                $contenidoHTML.= '<ul>';
                foreach ($parameters as $key => $val) {
                    if ($key === 'USERNAME' || $key === 'PASSWORD' || $val === '')
                        continue;
                    $contenidoHTML.=$this->format_error_msg($key, $val);
                }
                $contenidoHTML.= '</ul>';
            }
            echo $contenidoHTML;
            salir(CLX_NUM_ZERO);
        }
        echo $contenidoHTML;
    }

    /**
     * Performs the webservice method call - NEIE means No Exception If returned xml is Empty
     * @param string $methodname	Method name
     * @param array $parameters 	Optional method parameters
     * @return mixed $response
     */
    public function call_neie($methodname, $parameters = array()) {
        $contenidoHTML = '';
        if (empty($this->cnx))
            echo $contenidoHTML;
        salir($this->errors[CLX_NUM_ZERO]);
        try {
            echo $contenidoHTML;
            return $this->cnx->__call($methodname, $parameters);
        } catch (Exception $e) {
            if (!strpos($e->getMessage(), 'we got no XML document') === false) {
                return CLX_CHAR_EMPTY_STRING;
            } else {
                $contenidoHTML.= '<b>' . $e->getMessage() . '</b>';
                if (!empty($parameters)) {
                    $contenidoHTML.= '<ul>';
                    foreach ($parameters as $key => $val) {
                        if ($key === 'USERNAME' || $key === 'PASSWORD' || $val === '')
                            continue;
                        $contenidoHTML.= $this->format_error_msg($key, $val);
                    }
                    $contenidoHTML.= '</ul>';
                }
                echo $contenidoHTML;
                salir(CLX_NUM_ZERO);
            }
        }
        echo $contenidoHTML;
    }

    /**
     * Login into system
     * @param string $username
     * @param string $password
     */
    public function login($username, $password) {
        if ($this->loggedin)
            return true;
        if (empty($this->hasSession())) {
            $this->startSession();
        }
        ClxFreeReturn($this->call('clxLogin', array('USERNAME' => $username, 'PASSWORD' => $password)));
        $this->loggedin = true;
        return true;
    }

    /**
     * Logs out user form system
     */
    public function logout() {
        if ($this->hasSession()) {
            ClxFreeReturn($this->call('clxLogout'));
        }
        $this->setSessionId(false);
        $this->loggedin = false;
    }

    /**
     * Initialices user connection session
     */
    protected function startSession() {
        //Init session
        $sessionid = $this->call('clxGetSessionId');
        //Setting cookie for next responses
        //NOTE: only work is $server->setPersistence(SOAP_PERSISTENCE_SESSION); si set
        foreach ($this->cnx->_cookies as $key => $val) {
            $this->cnx->__setCookie($key, $val[CLX_NUM_ZERO]);
        }
        $this->setSessionId($sessionid);
    }

    /**
     * End user connection session
     */
    protected function endSession() {
        $this->logout();
    }

    /**
     * Sets session id
     */
    protected function setSessionId($sessionid) {
        if (empty($sessionid)) {
            $this->sessionid = '0';
        } else {
            $this->sessionid = $sessionid;
        }
    }

    /**
     * Return true if session has been initialiced
     */
    public function hasSession() {
        return (!empty($this->sessionid)) ? true : false;
    }

}
