<?php

/* * *****************************************************************************
 *                Authorize.net AIM Interface using CURL
 * ******************************************************************************
 *      Author:     Micah Carrick
 *      Email:      email@micahcarrick.com
 *      Website:    http://www.micahcarrick.com
 *
 *      File:       authorizenet.class.php
 *      Version:    1.0.1
 *      Copyright:  (c) 2005 - Micah Carrick
 *                  You are free to use, distribute, and modify this software
 *                  under the terms of the GNU General Public License.  See the
 *                  included license.txt file.
 *
 * ******************************************************************************
 *  REQUIREMENTS:
 *      - PHP4+ with CURL and SSL support
 *      - An Authorize.net AIM merchant account
 *      - (optionally) http://www.authorize.net/support/AIM_guide.pdf
 *
 * ******************************************************************************
 *  VERION HISTORY:
 *
 *      v1.0.1 [01.19.2006] - Fixed urlencode glitch (finally)
 *      v1.0.0 [04.07.2005] - Initial Version
 *
 * ******************************************************************************
 *  DESCRIPTION:
 *
 *      This class was developed to simplify interfacing a PHP script to the
 *      authorize.net AIM payment gateway.  It does not do all the work for
 *      you as some of the other scripts out there do.  It simply provides
 *      an easy way to implement and debug your own script.
 *
 * ******************************************************************************
 */

class Authorizenet {

    var $field_string;
    var $fields = array();
    var $response_string;
    var $response = array();
    var $card_present = false;
    var $gateway_url = 'https://secure.authorize.net/gateway/transact.dll';
    var $demo_gateway_url = 'https://test.authorize.net/gateway/transact.dll';
    var $use_demo_gateway_url = false;
    var $debug = false;
    var $verify_ssl = false; //Set to true to verify ssl certificate
    var $test_request = true;

    function __construct() {
        // some default values
        $this->add_field('x_version', '3.1');
        $this->add_field('x_delim_data', 'true');
        $this->add_field('x_delim_char', '|');
        $this->add_field('x_url', 'false');
        $this->add_field('x_type', 'AUTH_ONLY');
        $this->add_field('x_method', 'CC');
        $this->add_field('x_relay_response', 'false');
    }

    function add_field($field, $value) {
        // adds a field/value pair to the list of fields which is going to be
        // passed to authorize.net.  For example: 'x_version=3.1' would be one
        // field/value pair.  A list of the required and optional fields to pass
        // to the authorize.net payment gateway are listed in the AIM document
        // available in PDF form from www.authorize.net
        $this->fields[$field] = $value;
    }

    function process() {
        // This function actually processes the payment.  This function will
        // load the $response array with all the returned information.  The return
        // values for the function are:
        // 1 - Approved
        // 2 - Declined
        // 3 - Error
        // construct the fields string to pass to authorize.net
        foreach ($this->fields as $key => $value)
            $this->field_string .= $key . '=' . urlencode($value) . '&';
        // execute the HTTPS post via CURL
        if ($this->use_demo_gateway_url === true) {
            $ch = curl_init($this->demo_gateway_url);
        } else {
            $ch = curl_init($this->gateway_url);
        }
        curl_setopt($ch, CURLOPT_HEADER, CLX_NUM_ZERO);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, CLX_NUM_ONE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, rtrim($this->field_string, '& '));
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $this->verify_ssl);
        $this->response_string = urldecode(curl_exec($ch));
        if (curl_errno($ch)) {
            $this->response['Response Reason Text'] = curl_error($ch);
            return CLX_NUM_THREE;
        } else
            curl_close($ch);
        // load a temporary array with the values returned from authorize.net
        $temp_values = explode('|', $this->response_string);
        // load a temporary array with the keys corresponding to the values
        // returned from authorize.net (taken from AIM documentation)
        $temp_keys = array('Response Code', 'Response Subcode', 'Response Reason Code', 'Response Reason Text', 'Approval Code', 'AVS Result Code', 'Transaction ID', 'Invoice Number', 'Description', 'Amount', 'Method', 'Transaction Type', 'Customer ID', 'Cardholder First Name', 'Cardholder Last Name', 'Company', 'Billing Address', 'City', 'State', 'Zip', 'Country', 'Phone', 'Fax', 'Email', 'Ship to First Name', 'Ship to Last Name', 'Ship to Company', 'Ship to Address', 'Ship to City', 'Ship to State', 'Ship to Zip', 'Ship to Country', 'Tax Amount', 'Duty Amount', 'Freight Amount', 'Tax Exempt Flag', 'PO Number', 'MD5 Hash', 'Card Code (CVV2/CVC2/CID) Response Code',
            'Cardholder Authentication Verification Value (CAVV) Response Code'
        );
        // add additional keys for reserved fields and merchant defined fields
        for ($i = CLX_NUM_ZERO; $i <= CLX_NUM_27; $i++) {
            array_push($temp_keys, 'Reserved Field ' . $i);
        }
        $i = CLX_NUM_ZERO;
        while (sizeof($temp_keys) < sizeof($temp_values)) {
            array_push($temp_keys, 'Merchant Defined Field ' . $i);
            $i++;
        }
        // combine the keys and values arrays into the $response array.  This
        // can be done with the array_combine() function instead if you are using
        // php 5
        for ($i = CLX_NUM_ZERO; $i < sizeof($temp_values); $i++) {
            $this->response[$temp_keys[$i]] = $temp_values[$i];
        }
        // Return the response code.
        if ($this->card_present === true) {
            return $this->response['Response Subcode'];
        } else {
            return $this->response['Response Code'];
        }
    }

    function get_response_reason_text() {
        return $this->response['Response Reason Text'];
    }

    function dump_fields() {
        $contenidoHTML = '';
        // Used for debugging, this function will output all the field/value pairs
        // that are currently defined in the instance of the class using the
        // add_field() function.
        $contenidoHTML.= <<<HTML
	    <h3>authorizenet_class->dump_fields() Output:</h3>
	    <table width="95%" border="1" cellpadding="2" cellspacing="0">
            <tr>
               <td bgcolor="black"><b><font color="white">Field Name</font></b></td>
               <td bgcolor="black"><b><font color="white">Value</font></b></td>
            </tr>
HTML;
        foreach ($this->fields as $key => $value) {
            $contenidoHTML.= '<tr><td>' . $key . '</td><td>' . urldecode($value) . '&nbsp;</td></tr>';
        }
        $contenidoHTML.= '</table><br>';
        echo $contenidoHTML;
    }

    function dump_response() {
        $contenidoHTML = '';
        // Used for debuggin, this function will output all the response field
        // names and the values returned for the payment submission.  This should
        // be called AFTER the process() function has been called to view details
        // about authorize.net's response.
        $contenidoHTML.= <<<HTML
	      <h3>authorizenet_class->dump_response() Output:</h3>;
	    <table width='95%' border='1' cellpadding='2' cellspacing='0'>
            <tr>
               <td bgcolor='black'><b><font color=') . 'white'>Index&nbsp;</font></b></td>
               <td bgcolor='black'><b><font color='white'>Field Name</font></b></td>
               <td bgcolor='black'><b><font color='white'>Value</font></b></td>
            </tr>
	      <h3>authorizenet_class->dump_response() Output:</h3>;
	    <table width="95%" border="1" cellpadding="2" cellspacing="0">
            <tr>
               <td bgcolor="black"><b><font color="white">Index&nbsp;</font></b></td>
               <td bgcolor="black"><b><font color="white">Field Name</font></b></td>
               <td bgcolor="black"><b><font color="white">Value</font></b></td>
            </tr>
HTML;
        $i = CLX_INIT_ZERO;
        foreach ($this->response as $key => $value) {
            $contenidoHTML.=<<<HTML
		<tr>
                  <td valign="top" align="center">'$i</td>
                  <td valign='top'>''</td>
                  <td valign="top">' . $key.'</td>
                  <td valign="top">' . $value.'&nbsp;</td>
               </tr>
HTML;
            $i++;
        }
        $contenidoHTML.= '</table><br>';
        echo $contenidoHTML;
    }

    function getAutorizationCode() {
        return (array_key_exists('Approval Code', $this->response)) ? $this->response['Approval Code'] : '';
    }

    /**
     * Return Response Message Based on response code
     *
     * @param unknown_type $response
     */
    function processResponseMessage($response) {
        $contenidoHTML = '';
        // Process the payment and output the results
        switch ($response) {
            case CLX_NUM_ONE:  // Successs
                $contenidoHTML.= '<b>Success:</b><br>' .
                        $this->get_response_reason_text() . '<br><br>Details of the transaction are shown below...<br><br>';
                break;
            case CLX_NUM_TWO:  // Declined
                $contenidoHTML.= '<b>Payment Declined:</b><br>' .
                        $this->get_response_reason_text() . '<br><br>Details of the transaction are shown below...<br><br>';
                break;
            case CLX_NUM_THREE:  // Error
                $contenidoHTML.= '<b>Error with Transaction:</b><br>' .
                        $this->get_response_reason_text();
                '<br><br>Details of the transaction are shown below...<br><br>';
                break;
            default:
                break;
        }
        echo $contenidoHTML;
    }

    function operationTypes($type) {
	$retorno='';
        switch ($type) {
            case CLX_NUM_ZERO:
                $retorno='AUTH_CAPTURE'; //0>Purchase - AUTH_CAP
                break;
            case CLX_NUM_ONE:
                $retorno= 'AUTH_ONLY'; //1>Authorization - AUTH_ONLY
                break;
            case CLX_NUM_TWO:
                $retorno= 'CAPTURE_ON'; //2>Offline Sale - CAPTURE_ON
                break;
            case CLX_NUM_THREE:
                $retorno= 'CREDIT'; //3>Credit - CREDIT
                break;
            case CLX_NUM_FOUR:
                $retorno= 'PRIOR_AUTH_CAPTURE'; //4>Prior Auth. Capture - PRIOR_AUTH_CAPTURE
                break;
            case CLX_NUM_FIVE:
                $retorno= 'VOID'; //5>Void - VOID
                break;
            default:
                $retorno= $type;
                break;
        }
	return $retorno;
    }

    /**
     * CLX Transaction Status Codes
     *
     * @param unknown_type $response
     * @return unknown
     */
    function responseCode($response) {
        $return = 'PE'; //Pending
        // Process the payment and output the results
        switch ($response) {
            case CLX_NUM_ONE:  // Successs
                $return = 'AP';
                break;
            case CLX_NUM_TWO:  // Declined
                $return = 'DE';
                break;
            case CLX_NUM_THREE:  // Error
                $return = 'ER';
                break;
            default:
                break;
        }
        return $return;
    }

    //IMPLEMENTATIONS
    /**
     * Process Credit Card Payments
     *
     * @param string $connector_code
     * @param string $operation_type
     * @param string $firstname
     * @param string $last_name
     * @param string $address
     * @param string $city
     * @param string $state
     * @param string $zip
     * @param string $country
     * @param string $email
     * @param string $phone
     * @param string $card_num
     * @param string $amount
     * @param string $exp_date
     * @param string $card_cvv_code
     * @return unknown
     */
    function processCcPayment($card_present, $auth_code, $operation_type, $firstname, $lastname, $address, $city, $state, $zip, $country, $email, $phone, $card_num, $amount, $mmyy_exp_date, $card_cvv_code) {
        if ($card_present == CLX_NUM_ZERO) {
            $connector_code = 'AUTHNET';
            $this->add_field('x_version', '3.1');
        } else {
            $connector_code = 'AUTHNET_SWIPE';
            $this->card_present = true;
            $this->add_field('x_cpversion', '1.0');
            $this->add_field('x_market_type', '2'); //2 = Retail
            $this->add_field('x_device_type', '8'); //8 Website
            $this->add_field('x_track1', '');
            $this->add_field('x_track2', '');
            $this->add_field('x_response_format', '1'); //0 xml, 1 Deimited
        }
        //Get Connector Information
        $SKCO = getTool('SKCO');
        $cnx = $SKCO->getConnector($connector_code);
        //First Account
        $this->gateway_url = $cnx['IP_ADDRESS'];
        $this->add_field('x_login', $cnx['OTHER_USERNAME']); //provided by authorize.net
        $this->add_field('x_tran_key', $cnx['OTHER_PASSWORD']); //Up to 10 characters
        $this->add_field('x_type', $this->operationTypes($operation_type));
        if ($this->test_request === true) {
            $this->add_field('x_test_request', 'true');
        }
        $this->add_field('x_relay_response', 'false');
        $this->add_field('x_delim_data', 'true');
        $this->add_field('x_delim_char', '|');
        $this->add_field('x_encap_char', '');
        //Person information
        $this->add_field('x_first_name', $firstname);
        $this->add_field('x_last_name', $lastname);
        $this->add_field('x_address', $address);
        $this->add_field('x_city', $city);
        $this->add_field('x_state', $state);
        $this->add_field('x_zip', $zip);
        $this->add_field('x_country', $country);
        $this->add_field('x_email', $email);
        $this->add_field('x_phone', $phone);
        // Card Information
        $this->add_field('x_method', 'CC');
        if ($operation_type == CLX_NUM_TWO)
            $this->add_field('x_auth_code', $auth_code);
        if ($operation_type == CLX_NUM_FOUR)
            $this->add_field('x_trans_id', $auth_code);
        $this->add_field('x_card_num', $card_num);   // test successful visa
        $this->add_field('x_amount', $amount);
        $this->add_field('x_exp_date', $mmyy_exp_date);    // march of 2008
        $this->add_field('x_card_code', $card_cvv_code);    // Card CAVV Security code
        $result = $this->process();
        return $this->responseCode($result);
    }

}
